/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_ROADEDGESEGMENT_H
#define DPC_IMPL_TYPE_ROADEDGESEGMENT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint32.h"
#include "impl_type_uint16.h"
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "dpc/impl_type_linegeometry.h"

namespace dpc {
struct RoadEdgeSegment {
    ::UInt32 id{ 0U };
    ::UInt16 contributing_sensor{ 0U };
    ::UInt8 type{ 0U };
    ::Float height_above_ground{ 0.0F };
    ::dpc::LineGeometry line_geometry;
    ::UInt8 is_start_out_of_view{ 0U };
    ::UInt8 is_end_out_of_view{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(id);
        fun(contributing_sensor);
        fun(type);
        fun(height_above_ground);
        fun(line_geometry);
        fun(is_start_out_of_view);
        fun(is_end_out_of_view);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(id);
        fun(contributing_sensor);
        fun(type);
        fun(height_above_ground);
        fun(line_geometry);
        fun(is_start_out_of_view);
        fun(is_end_out_of_view);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("id", id);
        fun("contributing_sensor", contributing_sensor);
        fun("type", type);
        fun("height_above_ground", height_above_ground);
        fun("line_geometry", line_geometry);
        fun("is_start_out_of_view", is_start_out_of_view);
        fun("is_end_out_of_view", is_end_out_of_view);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("id", id);
        fun("contributing_sensor", contributing_sensor);
        fun("type", type);
        fun("height_above_ground", height_above_ground);
        fun("line_geometry", line_geometry);
        fun("is_start_out_of_view", is_start_out_of_view);
        fun("is_end_out_of_view", is_end_out_of_view);
    }

    friend bool operator==(const ::dpc::RoadEdgeSegment& lhs, const ::dpc::RoadEdgeSegment& rhs) noexcept
    {
        return (lhs.id == rhs.id) && (lhs.contributing_sensor == rhs.contributing_sensor) && (lhs.type == rhs.type) && (fabs(static_cast<double>(lhs.height_above_ground - rhs.height_above_ground)) < DBL_EPSILON) && (lhs.line_geometry == rhs.line_geometry) && (lhs.is_start_out_of_view == rhs.is_start_out_of_view) && (lhs.is_end_out_of_view == rhs.is_end_out_of_view);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_ROADEDGESEGMENT_H
